/* xdwapi sample 5

Function :
  Attaches the text annotations with date and hour at each page of the DocuWorks document.
How to use :
  xdwsmpl5 document

  document : DocuWorks document (in).
*/

#include <stdio.h>
#include <windows.h>

#include <xdw_api.h>
#include <xdwapian.h>

void print_error(int code)
{
	switch (code) {
	case XDW_E_NOT_INSTALLED:
		fprintf(stderr, "DocuWorks not installed.");
		break;
	case XDW_E_FILE_NOT_FOUND:
		fprintf(stderr, "File not found.");
		break;
	case XDW_E_ACCESSDENIED:
	case XDW_E_INVALID_NAME:
	case XDW_E_BAD_NETPATH:
		fprintf(stderr, "File not opened.");
		break;
	case XDW_E_BAD_FORMAT:
		fprintf(stderr, "File bad format.");
		break;
	case XDW_E_INVALID_ACCESS:
		fprintf(stderr, "Invalid access.");
		break;
	default:
		fprintf(stderr, "Error.");
		break;
	}
}

int add_annotation(XDW_DOCUMENT_HANDLE h, int page, char* string)
{
	// Attach a text annotation.
	XDW_ANNOTATION_HANDLE annoation;
	int api_result = XDW_AddAnnotation(
		h, XDW_AID_TEXT, page, 500, 500, NULL, &annoation, NULL);
	if (api_result < 0) return api_result;

	// Set the string to the text annotation.
	api_result = XDW_SetAnnotationAttribute(
		h, annoation, XDW_ATN_Text, XDW_ATYPE_STRING, string, 0, NULL);

	return api_result;
}

int main(int argc, char** argv)
{
	int api_result = 0;

	if (argc != 2) {
		fprintf(stderr, "How to use : xdwsmpl5 document");
		return 0;
	}

	char in_path[_MAX_PATH];
	_fullpath(in_path, argv[1], _MAX_PATH);

	// Open the document handle.
	XDW_DOCUMENT_HANDLE h = NULL;
	XDW_OPEN_MODE_EX mode = {
		sizeof(XDW_OPEN_MODE_EX), XDW_OPEN_UPDATE, XDW_AUTH_NODIALOGUE };
	api_result = XDW_OpenDocumentHandle(in_path, &h, (XDW_OPEN_MODE*)&mode);
	if (api_result < 0) {
		print_error(api_result);
		return 0;
	}

	// Use XDW_GetDocumentInformation to get the number of total pages.
	XDW_DOCUMENT_INFO info = { sizeof(XDW_DOCUMENT_INFO), 0 };
	XDW_GetDocumentInformation(h, &info);
	int last_page = info.nPages;

	// Get the current date and hour.
	char date[120], time[120], date_time[250];
	GetDateFormat(GetThreadLocale(), 0, NULL, NULL, date, 120);
	GetTimeFormat(GetThreadLocale(), 0, NULL, NULL, time, 120);
	wsprintf(date_time, "%s %s", date, time);

	// Attach an annotation at each page.
	for (int i = 1; i <= last_page; i++) {
		api_result = add_annotation(h, i, date_time);
		if (api_result < 0) {
			print_error(api_result);
			break;
		}
	}

	// Reflect the changes to the document.
	if (api_result >= 0) {
		api_result = XDW_SaveDocument(h, NULL);
	}

	// Close the document handle.
	XDW_CloseDocumentHandle(h, NULL);

	if (api_result < 0) return 0;

	return 1;
}
